from io import BytesIO
from django.conf import settings
from django.utils.deconstruct import deconstructible
from django.core.files.storage import Storage
from django.core.files.base import File
import requests


@deconstructible
class DariahStorage(Storage):

    def __init__(self):
        self._storage_url = settings.DARIAH_STORAGE_LOCATION
        
        response = requests.post('https://pdpdev.de.dariah.eu/oauth2/oauth2/token',
                                 data={'grant_type': 'client_credentials'},
                                 auth=(settings.DARIAH_STORAGE_USER, settings.DARIAH_STORAGE_SECRET))

        self._token = response.json()['access_token']
        print('dariahstorage token: ' + self._token)

    def exists(self, name):
        print("'exists' called for name: " + name)
        return False

    # here we already register the resource in dariahstorage
    def get_valid_name(self, name):
        print("'get_valid' calledfor name: " + name)
        response = requests.post(self._storage_url,
                                 headers={'Authorization': 'bearer ' +
                                          self._token, 'Content-type': 'text/plain'},
                                 data={'lalala': 'hu'})

        storage_id = response.headers['location'].rsplit('/', 1)[-1]
        print("return dariahstorageid: " + storage_id)
        return storage_id

    # TODO: id registered and belonging to user?
    def get_available_name(self, name, max_length=None):
        print("'get_available_name' called: " + name)
        # remove '/dataset' from path
        name = name.rsplit('/', 1)[-1]
        response = requests.head(self._storage_url + name,
                          headers={'Authorization': 'bearer ' + self._token})

        print(response.status_code)
        # TODO: return ok if ok ;-)
        print(response.status_code == requests.codes.ok)
        print(response.headers)

        return name

    def delete(self, name):
        print("'delete' called for: " + name)


    # what if name is created here, and get_available and get_valid return blabla?
    def _save(self, name, content):
        print('save called for: ' + name)
        print(content.file.content_type)
        print(content.file)
        response = requests.put(self._storage_url + name,
                                headers={'Authorization': 'bearer ' +
                                        self._token, 'Content-type': content.file.content_type},
                                data = content.read())
        # TODO: error handling
        print(response)

        return name

    def _open(self, name, mode='rb'):
        return DariahStorageFile(name, self, mode)

    def _read(self, name):
        response = requests.get(self._storage_url + name,
                                headers={'Authorization': 'bearer ' + self._token})
        return BytesIO(response.content)


    # should we raise a not implemented error, as the url is not accessible
    # without token?
    def url(self, name):
        print('url() for name: ' + name)
        return self._storage_url + name

class DariahStorageFile(File):

    def __init__(self, name, storage, mode):
        print('file init for name: ' + name)
        self._name = name
        self._storage = storage
        self._mode = mode # not used yet
        self.file = BytesIO()
        self._is_read = False

    def read(self, num_bytes=None):
        if not self._is_read:
            self.file = self._storage._read(self._name)
            self._is_read = True

        return self.file.read(num_bytes)
