# -*- coding: utf-8 -*-
"""
Django settings for daad_django project.

Generated by 'django-admin startproject' using Django 1.10.2.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

from django.utils.translation import ugettext_lazy as _
import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


LANGUAGES = (
    ('en', _('English')),
    ('ru', _('Russian')),
)
MODELTRANSLATION_DEFAULT_LANGUAGE = 'en'

LOCALE_PATHS = [
    BASE_DIR + '/locale',
    ]

# Application definition

INSTALLED_APPS = [
    'modeltranslation',
    #'grappelli',
 #   'threadedcomments',
    'fluent_comments',  # must be before django_comments
    'django_comments',
    'django.contrib.sites',
    'django.contrib.admin',
    'registration',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'corsheaders',
    'rest_framework',
    'rest_framework.authtoken',
    'django_extensions',
    'taggit',
    'taggit_serializer',
    #'taggit_bootstrap',
    'django_activeurl',
    'ckeditor',
    'ddusers',
    'django_date_extensions',
    'ajax_select',
    #'userena',
    #'guardian',
    #'easy_thumbnails',
    #'accounts',
    'intercoolerjs',
    'widget_tweaks',
    'crispy_forms',
    'django_summernote',
    'publications',
    #'publications_bootstrap',

]

CRISPY_TEMPLATE_PACK = 'bootstrap3'

COMMENTS_APP = 'fluent_comments' #'threadedcomments'

MIDDLEWARE = [
    'intercooler_helpers.middleware.HttpMethodOverride',
    'intercooler_helpers.middleware.IntercoolerData',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    #'solid_i18n.middleware.SolidLocaleMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'intercooler_helpers.middleware.IntercoolerRedirector',
#    'django_downloadview.SmartDownloadMiddleware',
]

AUTHENTICATION_BACKENDS = (
    'userena.backends.UserenaAuthenticationBackend',
    'guardian.backends.ObjectPermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
)

ANONYMOUS_USER_ID = 1
SITE_ID = 1

ROOT_URLCONF = 'dd_django.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'dd_django.wsgi.application'

AUTH_USER_MODEL = 'ddusers.User'

LOGIN_REDIRECT_URL = '/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'applogfile': {
            'level':'DEBUG',
            'class':'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'log/dd_django.log'),
            'maxBytes': 1024*1024*15, # 15MB
            'backupCount': 10,
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['applogfile'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_I18N_PATTERNS = True
# Optional. If you want to use redirects, set this to True
#SOLID_I18N_USE_REDIRECTS = False

USE_L10N = True

USE_TZ = True

CORS_ORIGIN_ALLOW_ALL = True

SUMMERNOTE_CONFIG = {
# Using Summernote Air-mode
    'airMode': False,#True,
# Change editor size
    'width': '100%',
    'height': '300%',

# Use proper language setting automatically (default)
#    'lang': None,

# Or, set editor language/locale forcely
#    'lang': 'ko-KR',

# Customize toolbar buttons
    'toolbar': [
        #['style', ['style']],
        ['style', ['bold', 'italic', 'underline', 'clear']],
        ['para', ['ul', 'ol', ]],
        ['insert', ['link']],
    ],
}

#DOWNLOADVIEW_BACKEND = "django_downloadview.apache.XSendfileMiddleware"

try:
    from .local_settings import *
except:
    print("no local settings found")
