from django.forms import ModelForm, TextInput, inlineformset_factory
from ddusers.models import User, DataSet, Topic, Project, Country, Institution, Affiliation, DataFile, Publication, DataSetPublication
from taggit.forms import *
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Field, Div, ButtonHolder, Submit, Button, HTML
from crispy_forms.bootstrap import FormActions, TabHolder, Tab
from django_comments.forms import CommentForm
#import taggit_bootstrap
from modeltranslation.forms import TranslationModelForm
from django_summernote.widgets import SummernoteWidget, SummernoteInplaceWidget
from ajax_select.fields import AutoCompleteSelectField, AutoCompleteSelectMultipleField

UPDATED_MSG = """
    {% if updated %}<div id='updated-msg' class='alert alert-success' ic-trigger-on='scrolled-into-view' ic-action='delay:2500;fadeOut;remove' role='alert'>
    <i class='fa fa-spinner fa-spin'></i><strong>{{ target|title }} saved</strong></div>{% endif %}
    {% if err %}<div id='updated-msg' class='alert alert-danger' role='alert'>
    <i class='fa fa-spinner fa-spin'></i><strong>Error! {{ target|title }} could not be saved. Error encountered in fields {% for field in err %} {{ field }}, {% endfor %}</strong></div> {% endif %}"""

ADD_EDIT_HEADING="<h2>{% if new %}Add{% else %}Edit{% endif %} {{ target|title }}</h2><hr>"
REQ_FIELD = "<p class='help-block'>[*] required field</p><hr>"

class AffiliationForm(ModelForm):
    class Meta:
        model = Affiliation
        exclude = ['user', 'institution' ]#'name_of_institution', 'place_of_institution', 'country_of_institution', 'website_of_institution']

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML( ADD_EDIT_HEADING ),
        Div(
            Div('name_of_institution', css_class='col-md-12', ),
            Div('website_of_institution', css_class='col-md-12', ),
            Div('place_of_institution', css_class='col-md-6', ),
            Div('country_of_institution', css_class='col-md-6', ),
            Div('position', css_class='col-md-10', ),
            Div('main', css_class='col-md-2', ),
            css_class='row',
        ),
        HTML( REQ_FIELD ),
        HTML("<span id='indicator' style='display:none'><i class='fa fa-spinner fa-spin'></i> Saving...</span>"),
        FormActions(
            Submit('save', 'Save {{ target }}', ic_indicator='#indicator'),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )

class InstitutionForm(ModelForm):
    class Meta:
        model = Institution
        fields = ['name', 'department', 'city', 'country']

class CountryForm(ModelForm):
    class Meta:
        model = User
        fields = ['countries',]

class UserForm(ModelForm):
    class Meta:
        model = User
        fields = ['image',]

class IcUserdataForm(ModelForm):
    #description = forms.CharField(widget=SummernoteInplaceWidget())
    class Meta:
        model = User
        fields = ['name_prefix', 'name_suffix', 'first_name', 'middle_name', 'last_name', 'email', 'research_profile_short', 'research_profile_full', 'external_profile']
    #research_profile_short = forms.CharField( widget=forms.Textarea)
    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML(ADD_EDIT_HEADING),
        Div(
            #Div('academic_title', css_class='col-md-3', ),
            Div('name_prefix', css_class='col-md-2', ),
            Div('first_name', css_class='col-md-2', ),
            Div('middle_name', css_class='col-md-2', ),
            Div('last_name', css_class='col-md-4', ),
            Div('name_suffix', css_class='col-md-2', ),
            css_class='row',
        ),
        Div(
            Div('research_profile_short', css_class='col-md-4', ),
            Div('research_profile_full', css_class='col-md-8', ),
            css_class='row',
        ),
        Div(
            Div('email', css_class='col-md-6', ),
            Div('external_profile', css_class='col-md-6', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        HTML("<span id='indicator' style='display:none'><i class='fa fa-spinner fa-spin'></i> Saving...</span>"),
        FormActions(
            Submit('save', 'Save {{ target }}', data_toggle="collapse", data_target="#collapse-userdata-edit",),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )

class ProjectForm(ModelForm):
    class Meta:
        model = Project
        fields = ['name', 'url',]

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML( ADD_EDIT_HEADING ),
        Div(
            Div('name', css_class='col-md-12', ),
            Div('url', css_class='col-md-12', ),
            css_class='row',
        ),
        HTML( REQ_FIELD ),
        FormActions(
            Submit('save', 'Save {{ target }}'),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )
class DataSetPublicationForm(ModelForm):
    class Meta:
       model = DataSetPublication
       fields = ['pub_type',]
    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        Div(
            Div('pub_type', css_class='col-md-12',),
            css_class='row',
        ),
    )


class PublicationForm(ModelForm):
    class Meta:
       model = Publication
       exclude = ['pdf',]
       #fields = ['type', 'authors', 'title', 'month', 'year', 'journal', 'book_title', 'publisher', 'institution', 'volume', 'number', 'pages', 'image', 'url', 'isbn', 'doi' ]
    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML( ADD_EDIT_HEADING ),
        Div(
            Div('type', css_class='col-md-4',),
            Div('doi', css_class='col-md-4', ),
            Div('isbn', css_class='col-md-4', ),
            Div('authors', css_class='col-md-4',),
            Div('title', css_class='col-md-4',),
            Div('url', css_class='col-md-4', ),
            css_class='row',
        ),
        Div(
            Div('journal', css_class='col-md-3', ),
            Div('book_title', css_class='col-md-3', ),
            Div('publisher', css_class='col-md-3', ),
            Div('place', css_class='col-md-3', ),
            css_class='row',
        ),
        Div(
            Div('volume', css_class='col-md-4', ),
            Div('number', css_class='col-md-4', ),
            Div('month', css_class='col-md-4', ),
            Div('year', css_class='col-md-4', ),
            Div('pages', css_class='col-md-4', wrapper_class="controls" ),
            Div('institution', css_class='col-md-4', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        FormActions(
            Submit('save', 'Save {{ target }}'),
            #Button('cancel', 'Cancel', data_toggle="collapse", data_target="#collapse-{{  object_type }}-{{ object.id }}"),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )


class DataSetForm(ModelForm):
    class Meta:
        model = DataSet
        exclude = ('group', 'categories', 'title', 'subtitle', 'description', 'user', 'institution', 'title_en', 'title_ru', 'subtitle_en', 'subtitle_ru', 'image', 'countries', 'link', 'license', 'publications', 'languages_of_data', 'keyword_tags', 'methods_of_data_analysis', 'methods_of_data_collection', )

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML(ADD_EDIT_HEADING),
        Div(
            Div('version', css_class='col-md-4', ),
            Div('creators', css_class='col-md-4', ),
            Div('data_repository', css_class='col-md-4', ),
            css_class='row',
        ),
        Div(
            Div('date_of_data_creation_from', css_class='col-md-3', ),
            Div('date_of_data_creation_to', css_class='col-md-3', ),
            Div('date_of_data_creation_text', css_class='col-md-6', ),
            css_class='row',
        ),
        Div(
            Div('time_period_from', css_class='col-md-3', ),
            Div('time_period_to', css_class='col-md-3', ),
            Div('time_period_text', css_class='col-md-6', ),
            css_class='row',
        ),
        Div(
            Div('sources_of_data', css_class='col-md-6', ),
            Div('datatypes', css_class='col-md-6', ),
            css_class='row',
        ),
        Div(
            Div('institutional_affiliation', css_class='col-md-6', ),
            Div('funding', css_class='col-md-6', wrapper_class="controls"),
            css_class='row',
        ),
        Div(
            Div('related_dataset_text', css_class='col-md-6', ),
            Div('related_dataset', css_class='col-md-6', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        FormActions(
            Submit('save', 'Save {{ target }}'),
            # Button('cancel', 'Cancel', data_toggle="collapse", data_target="#collapse-{{  object_type }}-{{ object.id }}"),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )

class DataSetLinkForm(ModelForm):
    class Meta:
        model = DataSet
        fields = ('link', )

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML(ADD_EDIT_HEADING),
        Div(
            Div('link', css_class='col-md-12', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        FormActions(
            Submit('save', 'Save {{ target }}'),
            # Button('cancel', 'Cancel', data_toggle="collapse", data_target="#collapse-{{  object_type }}-{{ object.id }}"),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )


class DataSetDescForm(ModelForm):
    class Meta:
        model = DataSet
        fields = ('title', 'subtitle', 'description', 'categories', )

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML(ADD_EDIT_HEADING),
        Div(
            Div('title', css_class='col-md-12', ),
            Div('subtitle', css_class='col-md-12', ),
            Div('description', css_class='col-md-8', ),
            Div('categories', css_class='col-md-4', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        FormActions(
            Submit('save', 'Save {{ target }}'),
            # Button('cancel', 'Cancel', data_toggle="collapse", data_target="#collapse-{{  object_type }}-{{ object.id }}"),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )

class DataSetImageUploadForm(ModelForm):
    remove_photo = forms.BooleanField(required=False)
    class Meta:
        model = DataSet
        fields = ('image', 'remove_photo')

class DataFileUploadForm(ModelForm):
    class Meta:
        model = DataFile
        fields = ('file', 'data_file_type')

    helper = FormHelper()
    helper.form_tag = False
    helper.layout = Layout(
        HTML(ADD_EDIT_HEADING),
        Div(
            Div('file', css_class='col-md-6', ),
            Div('data_file_type', css_class='col-md-6', ),
            css_class='row',
        ),
        HTML(REQ_FIELD),
        FormActions(
            Submit('save', 'Save {{ target }}'),
            # Button('cancel', 'Cancel', data_toggle="collapse", data_target="#collapse-{{  object_type }}-{{ object.id }}"),
        ),
        Div(
            HTML(
                UPDATED_MSG
            ),
        ),
    )


class TopicForm(ModelForm):
    class Meta:
        model = Topic
        fields = ['title_en', 'title_ru', 'description_en', 'description_ru']
        exclude = ()

