# -*- coding: utf-8 -*-

from rest_framework import mixins
from rest_framework import generics

from django.shortcuts import get_object_or_404, render, redirect
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt

from django.forms import modelformset_factory, inlineformset_factory
from django.http import HttpResponseRedirect, Http404, HttpResponse, FileResponse
from io import StringIO, BytesIO
from zipfile import ZipFile

from django.contrib.auth.models import Group
from django.forms import formset_factory

from ddusers.models import User #user
from ddusers.models import Institution #user
from ddusers.models import Project #user
from ddusers.models import DataSet #data
from ddusers.models import DataFile #data
from ddusers.models import Topic #data
from ddusers.models import Country
from ddusers.models import ProtestEvent
from ddusers.models import City
from ddusers.models import WebPage
from ddusers.models import Publication
from ddusers.models import AffiliationTagged
from ddusers.models import Affiliation #user
from ddusers.models import KeywordTags
from ddusers.models import KeywordTagged
from ddusers.models import LanguageTagged
from ddusers.models import AnalysisMethodsTagged
from ddusers.models import CollectionMethodsTagged
from ddusers.models import Category
from ddusers.models import Documentation
from ddusers.models import DataSetPublication


from ddusers.serializers import UserSerializer
from ddusers.serializers import InstitutionSerializer
from ddusers.serializers import ProjectSerializer
from ddusers.serializers import DataSetSerializer
from ddusers.serializers import TopicSerializer

from taggit.models import Tag

import tempfile
from weasyprint import HTML

from django_downloadview import ObjectDownloadView

from django.core.files.uploadedfile import SimpleUploadedFile
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test

from ddusers.forms import UserForm, DataSetForm, DataSetDescForm, TopicForm, ProjectForm, IcUserdataForm, CountryForm, InstitutionForm, AffiliationForm, \
    DataSetImageUploadForm, DataFileUploadForm, PublicationForm, DataSetPublicationForm

from django.contrib import messages
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.forms import PasswordChangeForm
from django.shortcuts import render, redirect

data_file_view = ObjectDownloadView.as_view(model=DataFile)

"""
Data file: download file view
"""
@login_required
def download_file(request, pk):
    datafile = DataFile.objects.get(id=pk).file
    response = FileResponse(datafile)
    response["Content-Disposition"] = "attachment; filename=%s" % (datafile,)
    return response

"""
User: change password
"""
@login_required
def change_password(request):
    if request.method == 'POST':
        form = PasswordChangeForm(request.user, request.POST)
        if form.is_valid():
            user = form.save()
            update_session_auth_hash(request, user)  # Important!
            messages.success(request, 'Your password was successfully updated!')
            #return redirect('data/change_password')
            return render(request, 'auth/change_password.html', {'form': form, 'user': user, 'updated': True, 'err':None, })
        else:
            messages.error(request, 'Please correct the error below.')
    else:
        form = PasswordChangeForm(request.user)
    return render(request, 'auth/change_password.html', {
        'form': form
    })

def start_page(request):
    countries = Country.objects.all().exclude(public=False)
    webpage = WebPage.objects.get(title='start')
    return render(request, 'index.html', {'countries':countries, 'webpage':webpage, })

"""
country views
"""
@login_required
def country_ic_change(request, country_id):
    user = User.objects.get(id=request.user.id)
    country = Country.objects.get(id=country_id)
    if country in user.get_countries():
        user.countries.add( country )
    else:
        user.countries.remove( country )
    return render(request, 'get_user_countries.html', {'user': user, 'countries':countries,})


@login_required
def get_user_countries_ic(request, pk=None):
    user = User.objects.get(id=request.user.id)
    all_countries = Country.objects.filter(public=True)
    countries = list()
    if pk:
        country = Country.objects.get(id=pk)
        if country in user.get_countries():
            user.countries.remove( country )
        else:
            user.countries.add( country )
        user.save()

    for ac in all_countries:
        if ac in user.get_countries():
            countries.append( (ac, True) )
        else:
            countries.append( (ac, False) )
    return render(request, 'get_user_countries.html', {'user': user, 'countries':countries,})

@login_required
def get_user_projects(request):
    user = User.objects.get(id=request.user.id)
    objects = user.get_projects()
    object_type = 'project'
    object_text = 'Add new Discuss Data related project...'
    edit_url = 'ddusers.project_edit'
    add_url = 'ddusers.project_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})

@login_required
def get_user_datasets(request):
    user = User.objects.get(id=request.user.id)
    objects = user.get_datasets()
    object_type = 'dataset'
    edit_url = 'ddusers.dataset_edit'
    add_url = 'ddusers.dataset_add'
    return render(request, 'data/get_user_datasets.html', {'user': user, 'objects':objects, 'edit_url': edit_url, 'add_url': add_url,  'object_type': object_type})

@login_required
def get_user_publications(request):
    user = User.objects.get(id=request.user.id)
    objects = user.publications.all()
    object_type = 'publication'
    object_text = 'Add new Discuss Data related publication...'
    edit_url = 'ddusers.publication_edit'
    add_url = 'ddusers.publication_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})
    #return render(request, 'data/get_user_publications.html', {'user': user, })

@login_required
def publication_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    publication = Publication.objects.get(id=pk)
    user.publications.remove(publication)
    objects = user.publications.all()
    object_type = 'publication'
    object_text = 'Add new Discuss Data related publication...'
    edit_url = 'ddusers.publication_edit'
    add_url = 'ddusers.publication_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})

@login_required
def project_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    project = Project.objects.get(id=pk)
    objects = Project.objects.filter(user=user)
    object_type = 'project'
    object_text = 'Add new Discuss Data related project...'
    edit_url = 'ddusers.project_edit'
    add_url = 'ddusers.project_add'
    if user == project.user:
        project.delete()
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url, 'object_type': object_type, 'object_text': object_text,})

@login_required
def country_ic_add(request):
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        add = request.POST.get('country-tag','')

        #print( "REMOVE", remove )
        #print( "ADD", add )
        # set user to authenticated user
        #if form.is_valid():
        #    # get tags from form
        #    interests = form.cleaned_data["interests"]
        #    # clear all tags from user
        #    user.interests.clear()
        #    # add tags from form to user
        if len(add) > 0:
            user.countries.add(add)

        return render(request, 'get_user_countries.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'get_user_countries.html', {'user': user,})

@login_required
def affiliation_add(request):
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        add = request.POST.get('affiliation-tag','')
        if len(add) > 0:
            user.affiliations.add(add)

        return render(request, 'get_user_affiliations.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'get_user_affiliations.html', {'user': user,})

#@login_required
#def affiliation_remove(request, pk):
#    user = User.objects.get(id=request.user.id)
#    tag = AffiliationTagged.objects.get(id=pk)
#    user.affiliations.remove(tag)
#    return render(request, 'get_user_affiliations.html', {'user': user, })

#def get_affiliations(request):
#    qs = AffiliationTagged.objects.all()
    #query = None
    #if query:
    #    return qs.filter(name__istartswith=query)
    #else:
    #return render(request, 'get_tags.html', {'qs': qs, })

@login_required
def tag_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    tag = KeywordTagged.objects.get(id=pk)
    user.interests.remove(tag)
    return render(request, 'get_user_interests.html', {'user': user, })

@login_required
def tag_add(request):
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        add = request.POST.get('interest-tag','').lower()
        if len(add) > 0:
            user.interests.add(add)

        return render(request, 'get_user_interests.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'get_user_interests.html', {'user': user,})

def get_object_tags(request, object_type):
    if object_type == 'language':
        qs = LanguageTagged.objects.all()
    if object_type == 'keyword':
        qs = KeywordTagged.objects.all()
    if object_type == 'methods_of_data_collection':
        qs = CollectionMethodsTagged.objects.all()
    if object_type == 'methods_of_data_analysis':
        qs = AnalysisMethodsTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_language_tags(request):
    qs = LanguageTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_keyword_tags(request):
    qs = KeywordTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_institutions(request):
    qs = Institution.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })


@login_required
def get_user_interests(request):
    user = User.objects.get(id=request.user.id)
    return render(request, 'get_user_interests.html', {'user': user, })

@login_required
def get_user_profile(request):
    user = User.objects.get(id=request.user.id)
    return render(request, 'get_user_profile.html', {'user': user, })


@login_required
def userdata_edit(request):
    err = None
    updated = False
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = IcUserdataForm(request.POST, request.FILES, instance=user)
        if form.is_valid():
            user = form.save()
            user.save()
            updated = True
        else:
            err = form.errors.as_data()
    else:
        user = User.objects.get(id=request.user.id)
        form = IcUserdataForm(instance=user)
    return render(request, 'project_edit.html', {'form': form, 'user': user, 'updated': updated, 'err': err, 'target': 'User Profile',})

@login_required
def publication_edit(request, pk=None):
    if pk:
        # existing publication?
        publication = Publication.objects.get(id=pk)
        new = False
    else:
        # new publication
        publication = Publication()
        new = True
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(request.POST, instance=publication)
        if form.is_valid():
            publication = form.save()
            user.publications.add(publication)
            return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'updated': True, 'err':None, 'new':new, })
        else:
            return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'updated': False, 'err':  form.errors.as_data(), 'new':new,})
    else:
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(instance=publication)
    return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'err':None, 'new':new,})

@login_required
def ic_get_dataset_info(request, pk):
    ds = DataSet.objects.get(id=pk)
    return render(request, 'data/ic_get_dataset_info.html', {'ds': ds, 'updated': True, 'err':None, 'type': 'description' })

@login_required
def ic_get_dataset_datafiles(request, pk):
    ds = DataSet.objects.get(id=pk)
    return render(request, 'data/ic_get_dataset_datafiles.html', {'ds': ds, 'updated': True, 'err':None, 'type': 'files' })

@login_required
def ic_get_dataset_metadata(request, pk):
    ds = DataSet.objects.get(id=pk)
    return render(request, 'data/ic_get_dataset_metadata.html', {'ds': ds, 'updated': True, 'err':None, 'type': 'metadata' })

@login_required
def ic_dataset_image(request, pk):
    ds = DataSet.objects.get(id=pk)
    return render(request, 'data/ic_dataset_image.html', {'ds': ds, })

@login_required
def ic_dataset_image_upload(request, pk):
    template = 'data/ic_dataset_edit.html'
    updated = False
    target = 'dataset-image'
    ds = DataSet.objects.get(id=pk)
    err = None
    if ds.user != request.user:
        raise Http404("No right to access this page")
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = DataSetImageUploadForm(request.POST, request.FILES, instance=ds)
        if form.is_valid():
            ds.user = user
            if pk:
                #ds.id = pk
                ds = form.save()
                #if form.cleaned_data["image"]:
                #    ds.image = form.cleaned_data["image"]
                ds.save()
        else:
            err = form.errors.as_data()
    else:
        print( "GET" )
        user = User.objects.get(id=request.user.id)
        err = None
        form = DataSetImageUploadForm(instance=ds)

    return render(request, template, {'ds': ds, 'form': form, 'user': user, 'updated': updated, 'err':err, 'target' : target, })

def create_pdf(request, ds):
    html = HTML(string=ds.generate_html_string(), base_url=request.build_absolute_uri())
    return html.write_pdf()


@login_required
def ds_get_zip(request, ds_pk):
    ds = DataSet.objects.get(id=ds_pk)
    in_memory = BytesIO()
    zip = ZipFile(in_memory, "a")
    zip.writestr(ds.get_pdf_file_name(), create_pdf(request, ds))
    for file in ds.get_datafiles():
        zip.writestr(file.name, file.file.read())
    # fix for Linux zip files read in Windows
    for file in zip.filelist:
        file.create_system = 0
    zip.close()

    response = HttpResponse(content_type="application/zip")
    response["Content-Disposition"] = "attachment; filename=%s" % (ds.get_zip_file_name(),)
    in_memory.seek(0)
    response.write(in_memory.read())

    return response

@login_required
def ds_get_pdf(request, ds_pk):
    ds = DataSet.objects.get(id=ds_pk)
    result = create_pdf(request, ds)
    # Creating http response
    response = HttpResponse(content_type='application/pdf;')
    response['Content-Disposition'] = 'inline; filename=%s-description-discuss_data.pdf' % (ds.title.replace(' ', '_'),)
    response['Content-Transfer-Encoding'] = 'binary'
    #with tempfile.NamedTemporaryFile(delete=True) as output:
    #    output.write(result)
    #    output.flush()
    #    output = open(output.name, 'r')
    response.write(result)

    return response

@login_required
def ic_ds_object_tags_add(request, ds_pk, object_type):
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        tag = request.POST.get('%s-tag' % (object_type,),'')
        if len(tag) > 0:
            if object_type == "language":
                ds.languages_of_data.add(tag)
                tags = ds.get_language_tags()
            if object_type == "keyword":
                ds.keyword_tags.add(tag)
                tags = ds.get_keyword_tags()
            if object_type == "methods_of_data_collection":
                ds.methods_of_data_collection.add(tag)
                tags = ds.get_methods_of_data_collection_tags()
            if object_type == "methods_of_data_analysis":
                ds.methods_of_data_analysis.add(tag)
                tags = ds.get_methods_of_data_analysis_tags()
    else:
        if object_type == "language":
            tags = ds.get_language_tags()
        if object_type == "keyword":
            tags = ds.get_keyword_tags()
        if object_type == "methods_of_data_collection":
            tags = ds.get_methods_of_data_collection_tags()
        if object_type == "methods_of_data_analysis":
            tags = ds.get_methods_of_data_analysis_tags()


    #return render(request, 'data/ic_ds_keyword_tags.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), })
    return render(request, 'data/tag_add_control.html', {'user': user, 'ds': ds, 'tags': tags, 'object':object_type,})

@login_required
def ic_ds_object_tags_remove(request, ds_pk, tag_pk, object_type):
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    user = User.objects.get(id=request.user.id)
    if object_type == "language":
        tag = LanguageTagged.objects.get(id=tag_pk)
        ds.languages_of_data.remove(tag)
        tags = ds.get_language_tags()
    if object_type == "keyword":
        tag = KeywordTagged.objects.get(id=tag_pk)
        ds.keyword_tags.remove(tag)
        tags = ds.get_keyword_tags()
    if object_type == "methods_of_data_collection":
        tag = CollectionMethodsTagged.objects.get(id=tag_pk)
        ds.methods_of_data_collection.remove(tag)
        tags = ds.get_methods_of_data_collection_tags()
    if object_type == "methods_of_data_analysis":
        tag = AnalysisMethodsTagged.objects.get(id=tag_pk)
        ds.methods_of_data_analysis.remove(tag)
        tags = ds.get_methods_of_data_analysis_tags()

    return render(request, 'data/tag_add_control.html', {'user': user, 'ds':ds, 'tags': tags, 'object':object_type, })


#@login_required
#def ic_ds_language_tags_add(request, ds_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.user != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    if request.method == 'POST':
#        add = request.POST.get('language-tag','')
#        if len(add) > 0:
#            ds.languages_of_data.add(add)
#    else:
#        pass
#    remove_url = 'ddusers.ic_ds_language_tags_remove'
#    get_url = 'ddusers.get_language_tags'
#    add_url = 'ddusers.ic_ds_language_tags_add'
#    #return render(request, 'data/ic_ds_keyword_tags.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), })
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds': ds, 'tags': ds.get_language_tags(), 'object':'language',  'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url,})



#@login_required
#def ic_ds_keyword_tags_add(request, ds_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.user != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    if request.method == 'POST':
#        add = request.POST.get('keyword-tag','')
#
#        if len(add) > 0:
#            ds.keyword_tags.add(add)
#    else:
#        pass
#    #return render(request, 'data/ic_ds_keyword_tags.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), })
#    remove_url = 'ddusers.ic_ds_keyword_tags_remove'
#    get_url = 'ddusers.get_keyword_tags'
#    add_url = 'ddusers.ic_ds_keyword_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds': ds, 'tags': ds.get_keyword_tags(), 'object':'keyword', 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })

#@login_required
#def ic_ds_keyword_tags_remove(request, ds_pk, keyword_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.user != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    tag = KeywordTagged.objects.get(id=keyword_pk)
#    ds.keyword_tags.remove(tag)
#    remove_url = 'ddusers.ic_ds_keyword_tags_remove'
#    get_url = 'ddusers.get_keyword_tags'
#    add_url = 'ddusers.ic_ds_keyword_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })

#@login_required
#def ic_ds_language_tags_remove(request, ds_pk, language_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.user != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    tag = LanguageTagged.objects.get(id=language_pk)
#    ds.languages_of_data.remove(tag)
#    remove_url = 'ddusers.ic_ds_language_tags_remove'
#    get_url = 'ddusers.get_language_tags'
#    add_url = 'ddusers.ic_ds_language_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds':ds, 'tags': ds.get_language_tags(), 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })

@login_required
def ic_manage_ds_country_tags(request, ds_pk, country_pk=None):
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    user = User.objects.get(id=request.user.id)
    all_countries = Country.objects.filter(public=True)
    countries = list()
    if country_pk:
        country = Country.objects.get(id=country_pk)
        if country in ds.get_countries():
            ds.countries.remove( country )
        else:
            ds.countries.add( country )
        ds.save()

    for ac in all_countries:
        if ac in ds.get_countries():
            countries.append( (ac, True) )
        else:
            countries.append( (ac, False) )
    return render(request, 'data/ic_manage_ds_country_tags.html', {'user': user, 'ds': ds, 'countries':countries,})


@login_required
def ic_dataset_desc_edit(request, pk=None):
    #template = 'data/ic_dataset_edit.html'
    template = 'project_edit.html'
    updated = False
    target = 'Dataset Description'

    if pk:
        ds = DataSet.objects.get(id=pk)
        if ds.user != request.user:
            raise Http404("No right to access this page")
    else:
        ds = DataSet()

    if request.method == 'POST':
        print( "POST" )
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = DataSetDescForm(request.POST, request.FILES, instance=ds)

        if form.is_valid():
            ds = form.save(commit=False)
            ds.user = user
            if pk:
                ds.id = pk
                ds.save()
                form.save_m2m()
            else:
                ds.save()
                form.save_m2m()
                return redirect('ddusers.dataset_edit', pk=ds.id )
            err = None
            updated = True
        else:
            err = form.errors.as_data()
    else:
        print( "GET" )
        user = User.objects.get(id=request.user.id)
        err = None
        form = DataSetDescForm(instance=ds)

    return render(request, template, {'ds': ds, 'form': form, 'user': user, 'updated': updated, 'err':err, 'target' : target, })

@login_required
def ic_dataset_external_link_edit(request, pk=None):
    template = 'project_edit.html'
    updated = False
    target = 'Dataset External Link'
    pass

@login_required
def ic_dataset_metadata_edit(request, pk=None):
    #template = 'data/ic_dataset_edit.html'
    template = 'project_edit.html'
    updated = False
    target = 'Dataset Metadata'

    if pk:
        ds = DataSet.objects.get(id=pk)
        if ds.user != request.user:
            raise Http404("No right to access this page")
    else:
        ds = DataSet()

    if request.method == 'POST':
        print( "POST" )
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = DataSetForm(request.POST, request.FILES, instance=ds)

        if form.is_valid():
            ds = form.save(commit=False)
            ds.user = user
            if pk:
                ds.id = pk
                ds.save()
                form.save_m2m()
            else:
                ds.save()
                form.save_m2m()
                return redirect('ddusers.dataset_edit', pk=ds.id )
            err = None
            updated = True
        else:
            err = form.errors.as_data()
    else:
        user = User.objects.get(id=request.user.id)
        err = None
        form = DataSetForm(instance=ds)
    return render(request, template, {'ds': ds, 'form': form, 'user': user, 'updated': updated, 'err':err, 'target' : target, })


@login_required
def dataset_edit(request, pk=None):
    if pk:
        template = 'data/dataset_edit.html'
        # existing dataset?
        print("User", request.user)
        ds = DataSet.objects.get(id=pk)
        form = DataSetImageUploadForm(request.POST, request.FILES)
        print("DS User", ds.user)
        ds_id = pk
        if ds.user != request.user:
            raise Http404("No right to access this page")
    else:
        # new project
        template = 'data/dataset_add.html'
        ds = DataSet()
        form = DataSetImageUploadForm(request.POST, request.FILES)
        ds_id = ""
        #ds_desc.dataset = dataset
    return render(request, template, {'ds': ds, 'ds_id': ds_id, 'err':None, 'form':form,})

@login_required
def ic_ds_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    ds = DataSet.objects.get(id=pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    ds.delete()
    objects = user.get_datasets()
    object_type = 'dataset'
    object_text = 'Add Dataset ...'
    edit_url = 'ddusers.dataset_edit'
    return render(request, 'data/get_user_datasets.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': edit_url,  'object_type': object_type, 'object_text': object_text,})

@login_required
def ic_get_ds_publications(request, pk):
    user = User.objects.get(id=request.user.id)
    ds = DataSet.objects.get(id=pk)
    objects = ds.get_dataset_publications()
    object_type = 'publication'
    object_text = 'Add Publications...'
    edit_url = 'ddusers.ic_ds_publication_edit'
    add_url = 'ddusers.ic_ds_publication_add'
    delete_url = 'ddusers.ic_ds_publication_remove'
    return render(request, 'get_user_objects.html', {'user': user, 'objects': objects, 'ds': ds, 'edit_url': edit_url,
                                                     'add_url': add_url, 'delete_url': delete_url,
                                                     'object_type': object_type, 'object_text': object_text})

@login_required
def ic_ds_publication_remove(request, ds_pk, ds_pub_pk):
    ds_pub = DataSetPublication.objects.get(id=ds_pub_pk)
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    user = User.objects.get(id=request.user.id)
    #publication = Publication.objects.get(id=df_pk)
    #ds_pub = DataSetPublication.objects.get(id=ds_pub_pk)
    ds_pub.delete()
    #ds.publications.remove(publication)
    objects = ds.get_dataset_publications()
    object_type = 'publication'
    object_text = 'Add new Publication related to this Dataset...'
    edit_url = 'ddusers.ic_ds_publication_edit'
    add_url = 'ddusers.ic_ds_publication_add'
    delete_url = 'ddusers.ic_ds_publication_remove'
    return render(request, 'get_user_objects.html', {'user': user, 'objects': objects, 'ds': ds, 'edit_url': edit_url,
                                                     'add_url': add_url, 'delete_url':delete_url, 'object_type': object_type,
                                                     'object_text': object_text, })

@login_required
def ic_ds_publication_edit(request, ds_pk, ds_pub_pk=None):
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    if ds_pub_pk:
        # existing publication?
        ds_pub = DataSetPublication.objects.get(id=ds_pub_pk)
        publication = ds_pub.publication
        new = False
    else:
        # new publication
        publication = Publication()
        ds_pub = DataSetPublication()
        ds_pub.dataset = ds
        new = True
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(request.POST, instance=publication, prefix='publication')
        sub_form = DataSetPublicationForm(request.POST, instance=ds_pub, prefix='ds_pub')
        if form.is_valid() and sub_form.is_valid():
            publication = form.save()
            ds_pub = sub_form.save(commit=False)  # cleaned_data('pub_type')
            ds_pub.publication = publication
            ds_pub.save()
            #ds.publications.add(publication)
            updated = True
            err = None
            objects = ds.get_dataset_publications()
            object_type = 'publication'
            object_text = 'Add new Publication related to this Dataset...'
            edit_url = 'ddusers.ic_ds_publication_edit'
            add_url = 'ddusers.ic_ds_publication_add'
            delete_url = 'ddusers.ic_ds_publication_remove'
            return render(request, 'get_user_objects.html',
                          {'user': user, 'objects': objects, 'ds': ds, 'edit_url': edit_url,
                           'add_url': add_url, 'delete_url': delete_url, 'object_type': object_type,
                           'object_text': object_text, })
        else:
            updated = False
            err = form.errors.as_data()
            err_sub = sub_form.errors.as_data()
            new = new
    else:
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(instance=publication, prefix='publication')
        sub_form = DataSetPublicationForm(instance=ds_pub, prefix='ds_pub')
        updated = False
        err = None
        err_sub = None
    return render(request, 'project_edit.html', {'target': 'publication', 'ds':ds, 'form': form, 'sub_form':sub_form, 'user': user, 'updated': updated, 'err':err, 'err_sub':err_sub, 'new':new,})



@login_required
def ic_get_ds_datafiles(request, pk):
    user = User.objects.get(id=request.user.id)
    ds = DataSet.objects.get(id=pk)
    objects = ds.get_datafiles()
    object_type = 'datafile'
    object_text = 'Add Datafiles...'
    edit_url = 'ddusers.ic_ds_datafile_edit'
    add_url = 'ddusers.ic_ds_datafile_add'
    delete_url = 'ddusers.ic_ds_datafile_remove'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'ds': ds, 'edit_url': edit_url,
            'add_url': add_url,  'delete_url': delete_url, 'object_type': object_type, 'object_text': object_text})
    # return render(request, 'get_user_objects.html', {'user': user, 'objects': objects, 'edit_url': edit_url, 'add_url': add_url,  'object_type': object_type})

@login_required
def ic_ds_datafile_remove(request, ds_pk, df_pk):
    user = User.objects.get(id=request.user.id)
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    datafile = DataFile.objects.get(id=df_pk)
    datafile.delete()
    objects = ds.get_datafiles()
    object_type = 'datafile'
    edit_url = 'ddusers.ic_ds_datafile_edit'
    add_url = 'ddusers.ic_ds_datafile_add'
    delete_url = 'ddusers.ic_ds_datafile_remove'
    #if user == affiliation.user:
    #    affiliation.delete()
    return render(request, 'get_ds_datafiles.html', {'user': user, 'objects':objects, 'datafile': datafile, 'ds': ds, 'edit_url': edit_url, 'add_url': add_url,  'delete_url': delete_url, 'object_type': object_type})


@login_required
def ic_ds_datafile_add(request, ds_pk):
    ds = DataSet.objects.get(id=ds_pk)
    if ds.user != request.user:
        raise Http404("No right to access this page")
    datafile = DataFile()

    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = DataFileUploadForm(request.POST, request.FILES)
        if form.is_valid():
            datafile.dataset = ds
            datafile.data_file_type = form.cleaned_data["data_file_type"]
            uploadedfile = form.cleaned_data['file']
            datafile.file = uploadedfile
            datafile.name = uploadedfile._name #"%s-%s" % (ds.title.replace(' ', '_'), uploadedfile._name)
            datafile.content_type = uploadedfile.content_type
            datafile.data_file_size = uploadedfile._size
            datafile.save()
            updated = True
            form_err = None
        else:
            updated = False
            form_err = form.errors.as_data()
    else:
        user = User.objects.get(id=request.user.id)
        form = DataFileUploadForm()
        #sub_form = InstitutionForm(prefix='inst', instance=institution)
        updated = False
        form_err = None
        #sub_form_err = None
    return render(request, 'project_edit.html', {'target': 'datafile', 'form': form, 'user': user, 'form_err':form_err, 'updated':updated})
    #return render(request, 'affiliation_edit.html', {'target': 'affiliation', 'form': form, 'sub_form':sub_form, 'user': user, 'form_err':form_err, 'sub_form_err':sub_form_err, 'updated':updated})

@login_required
def ic_datafile_edit(request, ds_pk, df_pk):
    datafile = DataFile.objects.get(id=df_pk)
    if datafile.get_user() != request.user:
        raise Http404("No right to access this page")
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = DataFileUploadForm(request.POST, request.FILES, instance=datafile)
        if form.is_valid():
            datafile.data_file_type = form.cleaned_data["data_file_type"]
            datafile.file = form.cleaned_data["file"]
            datafile.save()
            updated = True
            form_err = None
        else:
            updated = False
            form_err = form.errors.as_data()
    else:
        user = User.objects.get(id=request.user.id)
        form = DataFileUploadForm(instance=datafile)
        #sub_form = InstitutionForm(prefix='inst', instance=institution)
        updated = False
        form_err = None
        #sub_form_err = None
    return render(request, 'project_edit.html', {'target': 'datafile', 'form': form, 'user': user, 'form_err':form_err, 'updated':updated})


@login_required
def get_user_affiliations(request):
    user = User.objects.get(id=request.user.id)
    objects = user.get_affiliations()
    object_type = 'affiliation'
    object_text = 'Add new affiliation...'
    edit_url = 'ddusers.affiliation_edit'
    add_url = 'ddusers.affiliation_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url, 'add_url': add_url,  'object_type': object_type, 'object_text': object_text})

@login_required
def affiliation_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    affiliation = Affiliation.objects.get(id=pk)
    objects = Affiliation.objects.filter(user=user)
    object_type = 'affiliation'
    object_text = 'Add new affiliation...'
    edit_url = 'ddusers.affiliation_edit'
    add_url = 'ddusers.affiliation_add'
    if user == affiliation.user:
        affiliation.delete()
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url, 'object_type': object_type, 'object_text': object_text})

@login_required
def affiliation_edit(request, pk=None):
    form = AffiliationForm(prefix='aff')
    if pk:
        # existing affiliation?
        affiliation = Affiliation.objects.get(id=pk)
        institution = affiliation.institution
        if affiliation.user != request.user:
            raise Http404("No right to access this page")
        new = False
    else:
        # new project
        affiliation = Affiliation()
        new = True

    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = AffiliationForm(request.POST, request.FILES, prefix='aff', instance=affiliation)
        if form.is_valid(): #and sub_form.is_valid():
            affiliation = form.save(commit=False)
            affiliation.user = user
            affiliation.save()
            updated = True
            form_err = None
        else:
            updated = False
            form_err = form.errors.as_data()

    else:
        user = User.objects.get(id=request.user.id)
        form = AffiliationForm(prefix='aff', instance=affiliation)
        updated = False
        form_err = None
    return render(request, 'project_edit.html', {'target': 'affiliation', 'form': form, 'user': user, 'form_err':form_err, 'updated':updated, 'new':new,})
    #return render(request, 'affiliation_edit.html', {'target': 'affiliation', 'form': form, 'sub_form':sub_form, 'user': user, 'form_err':form_err, 'sub_form_err':sub_form_err, 'updated':updated})

@login_required
def project_edit(request, pk=None):
    if pk:
        # existing project?
        project = Project.objects.get(id=pk)
        if project.user != request.user:
            raise Http404("No right to access this page")
        new = False
    else:
        # new project
        project = Project()
        new = True

    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = ProjectForm(request.POST, request.FILES)
        if form.is_valid():
            project.name = form.cleaned_data["name"]
            project.url = form.cleaned_data["url"]
            project.user = user
            project.save()

            return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'updated': True, 'err':None, 'new':new, })
        else:
            return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'updated': False, 'err':  form.errors.as_data(), 'new':new,})
    else:
        user = User.objects.get(id=request.user.id)
        form = ProjectForm(instance=project)
    return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'err':None, 'new':new,})
    #else:
    #    error_msg = "You don't have the rights to edit this dataset"
    #    return render(request, 'data/dataset_edit.html', {'error_msg': error_msg, 'user': user})

def check_user_group(user, group_name):
    return user.groups.filter(name=group_name).exists()



#@login_required
#def dataset_edit(request, pk):
#    user = User.objects.get(id=request.user.id)
#    dataset = DataSet.objects.get(id=pk)
#    if user:
#        if request.method == 'POST':
#            # set user to authenticated user
#            user = User.objects.get(id=request.user.id)
#            dataset_form = DataSetForm(request.POST, request.FILES)
#            if dataset_form.is_valid():
#                print( dataset_form)
#            else:
#                print( "not valid")
#                print( dataset_form.errors.as_data())
#        else:
#            dataset_form = DataSetForm(instance=dataset)
#        return render(request, 'data/dataset_edit.html', {'dataset_form': dataset_form, 'user': user})
#    else:
#        error_msg = "You don't have the rights to edit this dataset"
#        return render(request, 'data/dataset_edit.html', {'error_msg': error_msg, 'user': user})
#

#@login_required
#def topic_edit(request):
#    if request.method == 'POST':
#        topic_form = TopicForm(request.POST)
#        if topic_form.is_valid():
#            title = topic_form.cleaned_data["topic"]
#            description = topic_form.cleaned_data["description"]
#            try:
#                topic = Topic.objects.get(title=title)
#            except:
#                topic = Topic()
#                topic.title = title
#                topic.description = description
#                topic.save()
#            return HttpResponseRedirect('/data/user/' + str(user.id))
#        else:
#            print "not valid"
#            print topic_form.errors.as_data()
#


@login_required
def topic_add(request):
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        topic_form = TopicForm(request.POST)
        if topic_form.is_valid():
            title_en = topic_form.cleaned_data["title_en"]
            title_ru = topic_form.cleaned_data["title_ru"]
            description_en = topic_form.cleaned_data["description_en"]
            description_ru = topic_form.cleaned_data["description_ru"]
            try:
                topic = Topic.objects.get(title=title_en)
            except:
                topic = Topic()
            topic.title_en = title_en
            topic.title_ru = title_ru
            topic.description_en = description_en
            topic.description_ru = description_ru
            topic.save()
            user.topics.add(topic)
            user.save()
            saved = True
            return render(request, 'topic_add.html', {'saved':saved, 'topic':topic})
        else:
            return HttpResponse("Form not valid")
    else:
        user = User.objects.get(id=request.user.id)
        topic_form = TopicForm()
    #return HttpResponse("No post data")
    return render(request, 'topic_add.html', {'topic_form': topic_form, 'user': user})

@login_required
def get_topics(request):
    topics = Topic.objects.all()
    return render(request, 'topic_list_ic.html', {'topics':topics,})

@login_required
def user_datasets(request):
    user = User.objects.get(id=request.user.id)
    return render(request, 'data/user_datasets.html', {'user': user})

@login_required
def user_edit(request):
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        user_form = UserForm(request.POST, request.FILES)
        if user_form.is_valid():
            if user_form.cleaned_data["image"]:
                user.image = user_form.cleaned_data["image"]
            user.save()

            return render(request, 'edit_user.html', {'user_form': user_form, 'user': user, 'updated': True, })
            #return HttpResponseRedirect('/data/user/' + str(user.id))
        else:
            print( "not valid")
            print( user_form.errors.as_data())
    else:
        user = User.objects.get(id=request.user.id)
        user_form = UserForm(instance=user)
    return render(request, 'edit_user.html', {'user_form': user_form, 'user': user})

#    user = User.objects.get(id=request.user.id)
#    return render(request, 'data/edit_user.html', {'user': user})

#@csrf_exempt
def user_list_ajax(request):
    queryset = User.objects.all()
    return render(request, 'user_list_ajax.html', {'queryset':queryset,})

def user_detail_ajax(request, pk):
    user = User.objects.get( id=pk )
    return render(request, 'user_detail_ajax.html', {'user':user,})

def user_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
            if request.user.is_authenticated:
                queryset = User.objects.exclude(access='HID').filter(countries=country_obj).exclude(is_active=False).exclude(first_name='').exclude(last_name='')
            else:
                queryset = User.objects.filter(access='PUB').filter(countries=country_obj).exclude(is_active=False).exclude(first_name='').exclude(last_name='')
        except IndexError:
            if request.user.is_authenticated:
                queryset = User.objects.exclude(access='HID').exclude(is_active=False).exclude(first_name='').exclude(last_name='')
            else:
                queryset = User.objects.filter(access='PUB').exclude(is_active=False).exclude(first_name='').exclude(last_name='')
    else:
        if request.user.is_authenticated:
            queryset = User.objects.exclude(access='HID').exclude(is_active=False).exclude(first_name='').exclude(last_name='')
        else:
            queryset = User.objects.filter(access='PUB').exclude(is_active=False).exclude(first_name='').exclude(last_name='')
    return render(request, 'user_list.html', {'queryset':queryset, 'pagetitle':'Researchers', 'country':country_obj, 'countries': countries})

#def user_list_country(request, country):
#    countries = Country.objects.all()
#    country_obj = Country.objects.get(slug=country)
#    queryset = User.objects.filter(public=True).filter(countries=country_obj)
#    return render(request, 'data/user_list.html', {'queryset':queryset, 'pagetitle': 'Researchers', 'country':country_obj,'countries' : countries,})

def user_detail(request, pk):
    try:
        if request.user.is_authenticated:
            user = User.objects.filter(id=pk).exclude(access='HID')[0]
        else:
            user = User.objects.filter(id=pk).filter(access='PUB')[0]
    except:
        raise Http404("Page not found.")
    return render(request, 'user_detail.html', {'user' : user, })

def project_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
            if request.user.is_authenticated:
                projects = Project.objects.exclude(access='HID').filter(countries=country_obj)
            else:
                projects = Project.objects.filter(access='PUB').filter(countries=country_obj)
        except IndexError:
            if request.user.is_authenticated:
                projects = Project.objects.exclude(access='HID')
            else:
                projects = Project.objects.filter(access='PUB')
    else:
        if request.user.is_authenticated:
            projects = Project.objects.exclude(access='HID')
        else:
            projects = Project.objects.filter(access='PUB')
    return render(request, 'data/project_list.html', {'country':country_obj, 'countries':countries, 'projects':projects, 'pagetitle':'Projects'})

def protest_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
            pes = ProtestEvent.objects.filter(country=country_obj)
        except IndexError:
            pes = ProtestEvent.objects.all()
    else:
        pes = ProtestEvent.objects.all()
    return render(request, 'data/protest_list.html', {'country':country_obj, 'countries':countries, 'pes':pes, 'pagetitle':'Protests'})

def protest_detail(request, pk):
    try:
        pe = ProtestEvent.objects.get(id=pk)
    except:
        raise Http404("Page not found.")
    return render(request, 'data/protest_detail.html', {'pe' : pe,})

def project_list_country(request, country):
    country_obj = Country.objects.get(slug=country)
    projects = Project.objects.filter(countries=country_obj)
    return render(request, 'data/project_list.html', {'projects':projects, 'pagetitle':'Projects'})

def project_detail(request, pk):
    try:
        if request.user.is_authenticated:
            project = Project.objects.filter(id=pk).exclude(access='HID')[0]
        else:
            project = Project.objects.filter(id=pk).filter(access='PUB')[0]
    except:
        raise Http404("Page not found.")
    return render(request, 'data/project_detail.html', {'project' : project,})

def category_list(request):
    categories = Category.objects.all()
    url_detail = 'ddusers.category_detail'
    return render(request, 'category_list.html', {'objects':categories, 'pagetitle':'Categories'})

def category_detail(request, pk):
    countries = Country.objects.all().exclude(public=False)
    category = get_object_or_404(Category, id=pk)
    return render(request, 'category_detail.html', {'object': category, 'countries' : countries})

def documentation_list(request):
    docu_general = Documentation.objects.filter(public=True, docu_type='GEN').order_by('title')
    docu_best_practices = Documentation.objects.filter(public=True, docu_type='BPR').order_by('title')
    docu_legal = Documentation.objects.filter(public=True, docu_type='LEG').order_by('title')
    object_url = 'ddusers.documentation_detail'
    return render(request, 'category_list.html', {'docu_general': docu_general, 'docu_best_practices': docu_best_practices,
                                                  'docu_legal': docu_legal, 'pagetitle':'Rules and Best Practices', 'object_url':object_url,})

def documentation_detail(request, pk=None, slug=None):
    if pk:
        documentation = get_object_or_404(Documentation, id=pk)
    if slug:
        documentation = get_object_or_404(Documentation, slug=slug)

    return render(request, 'category_detail.html', {'object' : documentation,})


def institution_list(request):
    countries = Country.objects.all()
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country)[0]
            institutions = Institution.objects.filter(public=True).filter(countries=country_obj)
        except IndexError:
            institutions = Institution.objects.filter(public=True)
    else:
        institutions = Institution.objects.filter(public=True)
    return render(request, 'institution_list.html', {'country':country_obj, 'countries':countries, 'institutions':institutions, 'pagetitle':'Institutions'})

def institution_detail(request, pk):
    institution = get_object_or_404(Institution, id=pk)

    return render(request, 'institution_detail.html', {'institution' : institution,})

def dataset_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        try:
            country = request.GET['country']
            try:
                country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
                datasets = DataSet.objects.filter(countries=country_obj)
            except IndexError:
                datasets = DataSet.objects.all()
        except:
            topic = request.GET['topic']
            datasets = DataSet.objects.all()
    else:
        datasets = DataSet.objects.all()
    return render(request, 'data/dataset_list.html', {'country': country_obj, 'countries':countries, 'datasets': datasets, 'pagetitle':'Datasets'})

@login_required(login_url='/login/')
def dataset_detail(request, pk):
    dataset = get_object_or_404(DataSet, id=pk)
    #dataset_description = DataSetDescription.objects.get(dataset=dataset)
    return render(request, 'data/dataset_detail.html', {'ds' : dataset, })

def handler404(request):
    response = render_to_response('404.html', {},
                                  context_instance=RequestContext(request))
    response.status_code = 404
    return response



"""
not used
"""

class TopicList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Topic.objects.all()
    serializer_class = TopicSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class TopicDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Topic.objects.all()
    serializer_class = TopicSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

class DataSetList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = DataSet.objects.all()
    serializer_class = DataSetSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class DataSetDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = DataSet.objects.all()
    serializer_class = DataSetSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class InstitutionList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Institution.objects.all()
    serializer_class = InstitutionSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class InstitutionDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Institution.objects.all()
    serializer_class = InstitutionSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class ProjectList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Project.objects.all()
    serializer_class = ProjectSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class ProjectDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Project.objects.all()
    serializer_class = ProjectSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

class UserList(mixins.ListModelMixin,
                        #mixins.CreateModelMixin,
                        generics.GenericAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

    #def post(self, request, *args, **kwargs):
    #    return self.create(request, *args, **kwargs)

class UserDetail(mixins.RetrieveModelMixin,
                        #mixins.UpdateModelMixin,
                        #mixins.DestroyModelMixin,
                        generics.GenericAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    #def put(self, request, *args, **kwargs):
    #    return self.update(request, *args, **kwargs)

    #def delete(self, request, *args, **kwargs):
    #    return self.destroy(request, *args, **kwargs)

