from fabric.api import env, local, lcd
from fabric.colors import red, green
from fabric.decorators import task, runs_once
from fabric.operations import prompt

import fileinput
import os
import random
import sys

VENV = 'venv_dd'

PROJ_ROOT = os.path.dirname(env.real_fabfile)
env.project_name = 'discussdata'
env.python = 'python3' if 'VIRTUAL_ENV' in os.environ else './{}/bin/python3'.format(VENV)



#@task
#def setup_deb_package():
#    if os.getenv('VIRTUAL_ENV') or hasattr(sys, 'real_prefix'):
#        abort(red('Deactivate any virtual environments before continuing.'))
#    make_settings()

@task
def setup():
    """
    Set up a local development environment
    This command must be run with Fabric installed globally (not inside a
    virtual environment)
    """
    if os.getenv('VIRTUAL_ENV') or hasattr(sys, 'real_prefix'):
        abort(red('Deactivate any virtual environments before continuing.'))
    make_settings()
    make_virtual_env()


    print ('\nDevelopment environment successfully created.\n' +
           'Create a Postgres database, enter its information into ' +
           'dd_django/local_settings.py, and run `fab sync_database` to finish.')

@task
@runs_once
def make_settings():
    """
    Generate a local settings file.
    Without any arguments, this file will go in editorsnotes/settings_local.py.
    If the function is passed an argument that defines env.hosts, this file will
    be placed in the deploy directory with the name settings-{host}.py
    """
    to_create = (['deploy/settings-{}.py'.format(host) for host in env.hosts]
                 or ['dd_django/local_settings.py'])

    for settings_file in to_create:
        secret_key = generate_secret_key()
        with lcd(PROJ_ROOT):
            local('if [ ! -f {0} ]; then cp {1} {0}; fi'.format(
                settings_file, 'dd_django/local_settings.py.example'))
            for line in fileinput.input(settings_file, inplace=True):
                print line.replace("SECRET_KEY = ''",
                                    "SECRET_KEY = '{}'".format(secret_key)),



def make_virtual_env():
    "Make a virtual environment for local dev use"
    with lcd(PROJ_ROOT):
        #local('virtualenv .')
        local('python3 -m venv {}'.format(VENV))
        local('./{}/bin/pip install -r requirements.txt'.format(VENV))


def generate_secret_key():
    SECRET_CHARS = 'abcdefghijklmnopqrstuvwxyz1234567890-=!@#$$%^&&*()_+'
    return ''.join([random.choice(SECRET_CHARS) for i in range(50)])


@task
def sync_database():
    "Sync db, make cache tables, and run South migrations"

    new_installation = len(get_db_tables()) == 0

    with lcd(PROJ_ROOT):
        print('\nWANT TO RUN MIGRATE STUFF.\n')
        local('{python} manage.py migrate'.format(**env))
#        local('{python} manage.py syncdb --noinput'.format(**env))
#        create_cache_tables()
#        local('{python} manage.py migrate reversion --noinput'.format(**env))
#        local('{python} manage.py migrate --noinput'.format(**env))
    if new_installation:
        print('\nDatabase synced.\n'+
              'run `fab create_superuser` to create a superuser and project.')


@task
def create_superuser():
    username = prompt('Username: ', validate=str)
    if username:
        local('{python} manage.py createsuperuser --username {username}'.format(
            username=username, **env))

    project_name = prompt('Project name (blank to skip): ')
    if project_name:
        project_slug = prompt('Project slug: ', validate=str)

    local('{python} manage.py createproject '
          '"{name}" {slug} --users={username}'.format(
              name=project_name, slug=project_slug, username=username, **env))


def get_db_tables():
    tables = local('{python} manage.py inspectdb | '
                   'grep "db_table =" || true'.format(**env), capture=True)
    return tables or []


